DAT programName         byte "EncoderDemo_131210k", 0
CON
{{
  ******* Public Notes *******

  Author: Duane Degn
  Date: December 10, 2013

  MIT License
  
  While there is motor control features in
  the object being demonstrated, this demo
  program doesn't not include a way to interface
  with the motor control aspects of the driver.
  
-------------------------------------------------  
  Motor Control Information:
-------------------------------------------------  
  The "FourQuadratureMotor" object can control
  motors requiring normal PWM input as well
  as continuous rotation servos.

  I have not tested the precision of the servo
  control signal. I doubt it has 1us resolution
  as the Servo32vX object does. If you have
  cogs available, you may wish to use a
  different object than "FourQuadratureMotor"
  for continuous rotation servo control.
  
  The program can not currently use both normal
  h-bridge motors and continuous rotation
  simultaneously. All motors are assumed to be 
  the same type. The program can use both
  types of direction control features. A
  h-bridge with single pin direction control
  may be used together with a dual pin control
  h-bridge.

  Set WHEEL_ENABLE_x to -1 if no motor used on
  that channel.
  
  If WHEEL_ENABLE_x if set to a valid I/O pin
  and WHEEL_RED_x is set to -1, the motor will
  be treated as a continuous rotation servo.

  If WHEEL_ENABLE_x and WHEEL_RED_x are both
  set to valid I/O pins and WHEEL_BLACK_x is
  set to -1, the WHEEL_RED_x pin will be used
  to set the direction of the motor. High for
  forward, low for reverse. Your h-bridge must
  support this type of interface in order to
  use it. This interface does not allow
  dynamic braking.
  
  If WHEEL_ENABLE_x, WHEEL_RED_x and
  WHEEL_BLACK_x are all set to valid I/O pins,
  WHEEL_RED_x and WHEEL_BLACK_x pins will be
  used to set the direction of the motor.
  Forward speeds will set WHEEL_RED_x high
  and WHEEL_BLACK_x low. The high and low
  states are switched for reverse.
  When speed is set to zero both direction
  pins will be set low.

-------------------------------------------------  
  Encoder Information:
-------------------------------------------------  
  Up to four quadrature encoders may be used
  with the "FourQuadratureMotor" object. The
  "A" and "B" sensor pins need to be
  consecutive I/O pins. Only the first of the
  two pins is listed in the "FIRST_ENCODER_x"
  section of the header object. Provisions
  have been made to reverse the direction
  of the encoder feedback if desired without
  the need to physically swap the wires
  connected to the Propeller. See comments
  within the program about setting the
  encoder direction.

  If an encoder is not used the "FIRST_ENCODER_x"
  pin should be set to -1.

  If an encoder pin is set to a valid I/O pin
  but the I/O pin is left to float, the input
  pins may register phantom transitions.

  There are several constants in the header
  object used to compute speeds.

  The constant "MAX_TRANSITIONS_PER_SEC" is
  used to scale the speed in order to have
  the measured speed have the same range as
  the PWM resolution. For example, if the
  "DEFALUT_PWM_RESOLUTION" constant in the
  child object is set to 1000, then a speed
  setting of 1000 should produce a measured
  speed of 1000. In order for the measured
  speed to match the input speed the constant
  "MAX_TRANSITIONS_PER_SEC" needs to be set
  to the appropriate value.
  
  This appropriate value may be calculated
  based on the motors known max rpm and
  transitions per revolution data or it
  may be directly measured by running this
  program while the motor runs at full
  speed. Currently, the motor will need to
  be powered from another source since this
  program does not yet have a way to input
  speed. 

  Rather than having a single constant for
  the number of transitions per revolution
  of the motor, I used two.
  "TRANS_PER_REV_NUMERATOR" and
  "TRANS_PER_REV_DENOMINATOR" 
  
}}
{
  ******* Private Notes *******

  131210b "RPM_SPEED_MULTIPLIER" no longer need
  now that "integerMathEnhancer[0]" is being used.
  "RPM_SPEED_MULTIPLIER" now causes 32-bit overflow.
  10c Is working better but only a tenth of an
  rpm unit of accuracy.
  10g Try to maximize resolution of speed.
  10g Some sort of timing problem. It's not
  registering stopped consistently.
  10h Seems to work okay.
  10i Add divide with rounding method.
  10i The divide with rounding didn't change the
  values much. If there was a difference it was
  only in the very last digit. I don't think
  rounding is worth the time delay.
  10j Remove rounding method and clean up
  formatting a bit.
  10j Spell check comments.
  10k Simplify a few constants.
  
}
CON                     
                     
  _clkmode = xtal1 + pll16x 
  _xinfreq = 5_000_000

  MAX_POSSIBLE_ENCODERS = Encoders#TOTAL_ENCODERS
  ARRAYS_TO_FREEZE = 3
  LONGS_TO_FREEZE = MAX_POSSIBLE_ENCODERS * ARRAYS_TO_FREEZE
  MAX_POSITIVE = $7FFFFFFF
  LARGEST_PRACTICAL_MULTIPLIER = MAX_POSITIVE 
  LARGEST_POWER_OF_TEN_LIKELY = 10000
  
VAR

  long directionCorrection[MAX_POSSIBLE_ENCODERS], output[MAX_POSSIBLE_ENCODERS]
  long zeroSpeedTime
  long newCount[MAX_POSSIBLE_ENCODERS], pulseToSpeedNumerator
  long transPerSecSpeed[MAX_POSSIBLE_ENCODERS], scaledSpeed[MAX_POSSIBLE_ENCODERS] 
  long rpmSpeed[MAX_POSSIBLE_ENCODERS]', linearSpeed[MAX_POSSIBLE_ENCODERS], linearUnitsPtr 
  long integerMathEnhancer[2], otherPortionOfEnhancer[2], powerOfTenPortionOfEnhancer[2]
  long powerOfTenPortionsOfEnhancer, otherPortionsOfEnhancer
  
  '' The following longs need to be kept in order and together.
  long frozenTransitionTime[MAX_POSSIBLE_ENCODERS], frozenLastPulse[MAX_POSSIBLE_ENCODERS]
  long frozenEncoderCount[MAX_POSSIBLE_ENCODERS]
  long previousTransitionTime[MAX_POSSIBLE_ENCODERS], previousLastPulse[MAX_POSSIBLE_ENCODERS]
  long previousEncoderCount[MAX_POSSIBLE_ENCODERS]
  long legacyLongsNotUsed[MAX_POSSIBLE_ENCODERS]
  long transitionTime[MAX_POSSIBLE_ENCODERS], timeOfLastPulse[MAX_POSSIBLE_ENCODERS]
  long totalEncoderCount[MAX_POSSIBLE_ENCODERS]  
  '' The above longs need to be kept in order and together.

  byte activeChannelFlag[MAX_POSSIBLE_ENCODERS], stillStoppedFlag[MAX_POSSIBLE_ENCODERS]
  
OBJ

  Pst : "Parallax Serial Terminal"
  Encoders : "FourQuadratureMotors131210a"            ' one PASM cog
  Header : "HeaderDemo_131209a"
  
PUB Main | frozenCnt, frozenIna

  directionCorrection[0] := Header#FORWARD0
  directionCorrection[1] := Header#FORWARD1
  directionCorrection[2] := Header#FORWARD2
  directionCorrection[3] := Header#FORWARD3
  '' "directionCorrection" allows direction reversal without requiring wires be physically moved.
  '' If the encoder direction is reversed from direction desired, change the "FORWARDx" to -1 
  '' in header object.
  '' Motor direction may be reversed by swapping "RED" and "BLACK" pin asignments in header
  '' object.
   
  'linearUnitsPtr := Header.GetLinearUnits
  
  pulseToSpeedNumerator := (clkfreq / Header#MAX_TRANSITIONS_PER_SEC) * Encoders#DEFALUT_PWM_RESOLUTION
  '' pulseToSpeedNumerator used to scale speed in attempt to get target speed and measured speed
  '' to match

  ComputerMathEnhancers
    
  zeroSpeedTime := pulseToSpeedNumerator + (pulseToSpeedNumerator / 2)
  '' How long a pulse needs to be to be considered stopped.
  
  Pst.Start(115200)
  
  Encoders.StartEncoders(Header.GetEncoderPins, @totalEncoderCount, @legacyLongsNotUsed, Header.GetEnablePins, {
  } Header.GetRedPins, Header.GetBlackPins, @output, @directionCorrection)

  'waitcnt(clkfreq * 2 + cnt)
  
  Pst.Clear
    
  'refreshInterval := clkfreq / Header#REFRESH_RATE
  'nextRefreshTime := cnt + refreshInterval
  
  repeat
    Pst.Home
    Pst.ClearEnd
    Pst.Str(string(13, "        name of program = "))
    Pst.Str(@programName)
    Pst.Str(string(13, " name of encoder object = "))
    Pst.Str(Encoders.GetObjectName)
    Pst.Str(string(13, "  name of header object = "))
    Pst.Str(Header.GetObjectName)
      
    'Pst.Str(string(13, " pulseToSpeedNumerator = "))
    'Pst.Dec(pulseToSpeedNumerator)
  
    longmove(@previousTransitionTime, @frozenTransitionTime, LONGS_TO_FREEZE)
    longmove(@frozenTransitionTime, @transitionTime, LONGS_TO_FREEZE)
    '' Encoder data is copied to "frozen" arrays so the values don't change
    '' while being compared. This insures the "previous" value stored is
    '' the same value used in the comparison.
    
    frozenCnt := cnt
    repeat result from 0 to 3
      ifnot activeChannelFlag[result]
        next
      'Pst.Str(string("  , ["))
      'Pst.Dec(result)
      'Pst.Str(string("] "))
      stillStoppedFlag[result] := 0
      if scaledSpeed[result] == 0
        if timeOfLastPulse[result] == previousLastPulse[result]
          'Pst.Str(string("still zero"))
          'frozenLastPulse[result] :=  (frozenCnt - zeroSpeedTime) '- 1
          ' I'm not sure if the stopped encoder is handled correctly.
          stillStoppedFlag[result] := 1
        else
          'Pst.Str(string("  new zero"))
      else
        'Pst.Str(string("  not zero"))    
    'Pst.ClearEnd

    'DisplayEnhancers
  
    repeat result from 0 to 3
      if frozenCnt - frozenLastPulse[result] < zeroSpeedTime and stillStoppedFlag[result] == 0
        transPerSecSpeed[result] := clkfreq / frozenTransitionTime[result]
        rpmSpeed[result] := (((((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
          } integerMathEnhancer[0]) / frozenTransitionTime[result]) * {
          } 60) * integerMathEnhancer[1]) / Header#TRANS_PER_REV_NUMERATOR) / otherPortionsOfEnhancer
        scaledSpeed[result] := (pulseToSpeedNumerator / frozenTransitionTime[result]) * directionCorrection[result]
      else
        ' slower than allowed
        transPerSecSpeed[result] := 0
        rpmSpeed[result] := 0
        scaledSpeed[result] := 0
      ifnot activeChannelFlag[result]
        if timeOfLastPulse[result]
          activeChannelFlag[result] := 1
        else
          next
      newCount[result] := frozenEncoderCount[result] - previousEncoderCount[result] 
      Pst.NewLine
      Pst.ClearEnd
      'Pst.Str(string(13, "       newCount["))
      'Pst.Dec(result)
      'Pst.Str(string("] = "))
      'Pst.Dec(newCount[result])
      'Pst.ClearEnd
      Pst.Str(string(13, "     totalCount["))
      Pst.Dec(result)
      Pst.Str(string("] = "))
      Pst.Dec(frozenEncoderCount[result])
      Pst.ClearEnd
      Pst.Str(string(13, " transitionTime["))
      Pst.Dec(result)
      Pst.Str(string("] = "))
      Pst.Dec(frozenTransitionTime[result])
      Pst.Str(string(" clock cycles, average of "))
      Pst.Dec(Encoders#BUFFER_SIZE_ENCODER_TIME)
      Pst.Str(string(" readings"))
      Pst.ClearEnd
      Pst.Str(string(13, "          speed["))
      Pst.Dec(result)
      Pst.Str(string("] = "))
      Pst.Dec(transPerSecSpeed[result])
      Pst.ClearEnd 
      Pst.Str(string(13, "                   (transitions per second)"))
      Pst.ClearEnd
      Pst.NewLine 
      Pst.ClearEnd 
      Pst.Str(string(13, "    scaledSpeed["))
      Pst.Dec(result)
      Pst.Str(string("] = "))
      Pst.Dec(scaledSpeed[result])
      Pst.ClearEnd
      Pst.Str(string(13, " scaled to full speed of "))
      Pst.Dec(Encoders#DEFALUT_PWM_RESOLUTION) 
      Pst.Str(string(" (current PWM resolution)"))
      Pst.ClearEnd
      Pst.NewLine
      Pst.ClearEnd
      Pst.Str(string(13, " rpm = "))
      DecPoint(rpmSpeed[result], powerOfTenPortionsOfEnhancer)
      Pst.ClearEnd
      Pst.Str(string(13, " Based on "))
      Pst.Dec(Header#TRANS_PER_REV_NUMERATOR) 
      if Header#TRANS_PER_REV_DENOMINATOR <> 1
        Pst.Str(string(" / "))
        Pst.Dec(Header#TRANS_PER_REV_DENOMINATOR)
      Pst.Str(string(" transitions per revolution."))
      Pst.ClearEnd
    
      'StepByStepDebug(result)
          
   
    Pst.NewLine
    Pst.ClearEnd
    Pst.str(string(13, "             I/O Pin Numbers"))
    Pst.ClearEnd
    Pst.str(string(13, "      tens:   3           2            1"))
    Pst.ClearEnd
    Pst.str(string(13, "     units:  1098_7654_3210_9876_5432_1098_7654_3210"))
    Pst.ClearEnd
    frozenIna := ina
    Pst.str(string(13, " ina state: "))
    ReadableBin(frozenIna)
    Pst.ClearEnd
    waitcnt(clkfreq / 10 + cnt) 
                              
PUB ComputerMathEnhancers
'' This method is used to find variables to optimize the precision
'' of the integer math.
'' These variables are not required.

  integerMathEnhancer[0] := (LARGEST_PRACTICAL_MULTIPLIER / clkfreq) / Header#TRANS_PER_REV_DENOMINATOR
  '' The pulse number used in the speed equations can result in significant rounding
  '' errors do to integer math. This factor is used to reduce the rounding error by
  '' increasing the original value to near the point of 32-bit(signed) rollover.
  
  powerOfTenPortionOfEnhancer[0] := LARGEST_POWER_OF_TEN_LIKELY
  repeat
    otherPortionOfEnhancer[0] := integerMathEnhancer[0] / powerOfTenPortionOfEnhancer[0]
    if otherPortionOfEnhancer[0] == 0
      powerOfTenPortionOfEnhancer[0] /= 10
  while otherPortionOfEnhancer[0] == 0 and powerOfTenPortionOfEnhancer[0] > 1
  if otherPortionOfEnhancer[0] == 0
    otherPortionOfEnhancer[0] := 1

  integerMathEnhancer[0] := otherPortionOfEnhancer[0] * powerOfTenPortionOfEnhancer[0]

  result := clkfreq / ((Header#MAX_TRANSITIONS_PER_SEC * 3) / 2) ' shortest possible transition time
  integerMathEnhancer[1] := result / 60 
  '' This factor is used to reduce the rounding error by
  '' increasing the intermediate value to near the point of 32-bit(signed) rollover.
  
  powerOfTenPortionOfEnhancer[1] := LARGEST_POWER_OF_TEN_LIKELY
  repeat
    otherPortionOfEnhancer[1] := integerMathEnhancer[1] / powerOfTenPortionOfEnhancer[1]
    if otherPortionOfEnhancer[1] == 0
      powerOfTenPortionOfEnhancer[1] /= 10
  while otherPortionOfEnhancer[1] == 0 and powerOfTenPortionOfEnhancer[0] > 1
  if otherPortionOfEnhancer[1] == 0
    otherPortionOfEnhancer[1] := 1
  
  integerMathEnhancer[1] := otherPortionOfEnhancer[1] * powerOfTenPortionOfEnhancer[1]

  powerOfTenPortionsOfEnhancer := powerOfTenPortionOfEnhancer[0] * powerOfTenPortionOfEnhancer[1]
  otherPortionsOfEnhancer := otherPortionOfEnhancer[0] * otherPortionOfEnhancer[1]
         
PUB DisplayEnhancers

  Pst.Str(string(13, " LARGEST_PRACTICAL_MULTIPLIER = "))
  Pst.Dec(LARGEST_PRACTICAL_MULTIPLIER)
  Pst.ClearEnd
  Pst.Str(string(13, " integerMathEnhancer[0] = "))
  Pst.Dec(integerMathEnhancer[0])
  Pst.ClearEnd
  Pst.Str(string(13, " otherPortionOfEnhancer[0] = "))
  Pst.Dec(otherPortionOfEnhancer[0])
  Pst.ClearEnd
  Pst.Str(string(13, " powerOfTenPortionOfEnhancer[0] = "))
  Pst.Dec(powerOfTenPortionOfEnhancer[0])
  Pst.ClearEnd
  Pst.Str(string(13, " integerMathEnhancer[1] = "))
  Pst.Dec(integerMathEnhancer[1])
  Pst.ClearEnd
  Pst.Str(string(13, " otherPortionOfEnhancer[1] = "))
  Pst.Dec(otherPortionOfEnhancer[1])
  Pst.ClearEnd
  Pst.Str(string(13, " powerOfTenPortionOfEnhancer[1] = "))
  Pst.Dec(powerOfTenPortionOfEnhancer[1])
  Pst.ClearEnd
  Pst.Str(string(13, " powerOfTenPortionsOfEnhancer = "))
  Pst.Dec(powerOfTenPortionsOfEnhancer)
  Pst.ClearEnd
  Pst.Str(string(13, " otherPortionsOfEnhancer = "))
  Pst.Dec(otherPortionsOfEnhancer)
  Pst.ClearEnd

PUB ReadableBin(localValue)

  Pst.char("%")
  Pst.bin(localValue >> 28, 4)
  Pst.char("_")
  Pst.bin(localValue >> 24, 4)
  Pst.char("_")
  Pst.bin(localValue >> 20, 4)
  Pst.char("_")
  Pst.bin(localValue >> 16, 4)
  Pst.char("_")
  Pst.bin(localValue >> 12, 4)
  Pst.char("_")
  Pst.bin(localValue >> 8, 4)
  Pst.char("_")
  Pst.bin(localValue >> 4, 4)
  Pst.char("_")
  Pst.bin(localValue, 4)

PUB DecPoint(value, denominator)
  if value < 0
    Pst.Char("-")
    -value
      
  if value => denominator
    result := value / denominator
    Pst.Dec(result)
    value //= denominator     
  else    
    Pst.Char("0")
  Pst.Char(".")  
  repeat while denominator > 1
    denominator /= 10
    if value => denominator
      result := value / denominator
      Pst.Dec(result)
      value //= denominator
    else
      Pst.Char("0")
      
PUB StepByStepDebug(localIndex)
'' Used to catch 32-bit overflow.

  Pst.Str(string(13, "clkfreq * Header#TRANS_PER_REV_DENOMINATOR = "))
  Pst.Dec(clkfreq * Header#TRANS_PER_REV_DENOMINATOR)
  Pst.ClearEnd
  Pst.Str(string(13, "clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0] = "))
  Pst.Dec(clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0])
  Pst.ClearEnd
  Pst.Str(string(13, "(clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0]) "))
  Pst.Str(string("/ frozenTransitionTime[result] = "))
  Pst.Dec((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0]) / frozenTransitionTime[localIndex])
  Pst.ClearEnd
  Pst.Str(string(13, "((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0]) "))
  Pst.Str(string("/ frozenTransitionTime[result]) * 60 = "))
  Pst.Dec(((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0]) / frozenTransitionTime[localIndex]) * {
      } 60)
  Pst.ClearEnd
  Pst.Str(string(13, "(((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0]) "))
  Pst.Str(string("/ frozenTransitionTime[result]) * 60) * integerMathEnhancer[1] = "))
  Pst.Dec((((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0]) / frozenTransitionTime[localIndex]) * {
      } 60) * integerMathEnhancer[1])
  Pst.ClearEnd
  Pst.Str(string(13, "(((((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0]) "))
  Pst.Str(string("/ frozenTransitionTime[result]) * 60) * integerMathEnhancer[1]) / "))
  Pst.Str(string("Header#TRANS_PER_REV_NUMERATOR) / otherPortionsOfEnhancer = "))
  Pst.Dec((((((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0]) / frozenTransitionTime[localIndex]) * {
      } 60) * integerMathEnhancer[1]) / Header#TRANS_PER_REV_NUMERATOR) / otherPortionsOfEnhancer)
  Pst.ClearEnd

PRI ExtraDebug
'' Not used.

  {Pst.Str(string(13, "    linearSpeed["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(linearSpeed[result])
  Pst.ClearEnd
  Pst.Str(string(13, " based on wheel diameter of "))
  Pst.Dec(Header#DIAMETER_OF_WHEEL) 
  Pst.Str(Header.GetLinearUnits)
  Pst.ClearEnd      }
   
  Pst.Str(string(13, "previousTransitionTime["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(previousTransitionTime[result])
  Pst.ClearEnd
  Pst.Str(string(13, "previousLastPulse["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(previousLastPulse[result])
  Pst.ClearEnd
  Pst.Str(string(13, "previousEncoderCount["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(previousEncoderCount[result])
  Pst.ClearEnd
  Pst.Str(string(13, "frozenTransitionTime["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(frozenTransitionTime[result])
  Pst.ClearEnd
  Pst.Str(string(13, "frozenLastPulse["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(frozenLastPulse[result])
  Pst.ClearEnd
  Pst.Str(string(13, "previousEncoderCount["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(frozenEncoderCount[result])
  Pst.ClearEnd
  Pst.Str(string(13, "legacyLongsNotUsed["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(legacyLongsNotUsed[result])
  Pst.ClearEnd
  Pst.Str(string(13, "transitionTime["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(transitionTime[result])
  Pst.ClearEnd
  Pst.Str(string(13, "timeOfLastPulse["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(timeOfLastPulse[result])
  Pst.ClearEnd
  Pst.Str(string(13, "totalEncoderCount["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(totalEncoderCount[result])
  Pst.ClearEnd
                      