DAT objectName          byte "FourQuadratureMotors131210a", 0
CON
{{
  ******* Public Notes *******

  This program is used to monitor four quadrature
  encoders like those used by the Rover 5 chassis.
  See comments at start of each method.

  This object was written by Duane Degn

  The code based on a the encoder technique shown in a
  Nuts & Volts article by Jon "JonnyMac" McPhalen.
  
}}
{
  ******* Private Notes *******
  
  FourQuadratureEncoders130125a modifed from VexMecanumRc121219b
  130125a Start removing non-encoder code from program and
  make this object a child object.
  0205a Add additional comments in preperation to posting to forum.
  15a Change to pointer to pins so pairs can be in any order.
  0314a Start adding ability to time pulses.
  131126a Add motor control masks.
  27d Kind of works with 27d parent and 27c grandparent.
  28a Kind of works with 28a parent and 28b grandparent.
  Change name from "FourQuadratureEncoders131128c" to
  "FourQuadratureMotors131128a".
  28b Abandon 28a. I changed too many things at once.
  1207a Try adding servo control.

}
CON

  TOTAL_ENCODERS = 4            ' This is not user changable. The other constants may
                                ' adjusted by user.
  MAX_ENCODER_INDEX = TOTAL_ENCODERS - 1                              
  SPEED_BUFFER_SIZE = 16
  'BUFFER_SIZE_ENCODER_TIME = SPEED_BUFFER_SIZE
  CYCLES_PER_ENCODER_CHECK = 1600 '800
  ENCODER_LOOPS_PER_WRITE = 126 '512
  DEFAULT_PWM_FREQUENCY = 200
  DEFALUT_PWM_RESOLUTION = 1000

  BITS_TO_AVERAGE_ENCODER_TIME = 6 '4  
 
  BUFFER_SIZE_ENCODER_TIME = 1 << BITS_TO_AVERAGE_ENCODER_TIME
 
  MAX_INDEX_ENCODER_TIME = BUFFER_SIZE_ENCODER_TIME - 1

  SERVO_FREQUENCY = 50
  SERVO_STOP = 1500
  SERVO_FULL_SPEED = 145 '500
    
VAR

  long speedBuffer[SPEED_BUFFER_SIZE * TOTAL_ENCODERS]
  long debugTime
  long pulseBuffer[BUFFER_SIZE_ENCODER_TIME * TOTAL_ENCODERS]
  long pwmOnTime[TOTAL_ENCODERS], pwmTime
  long pwmFrequency, pwmResolution
  long targetPtr, negativePinPtr, positivePinPtr
  long largestMotorsInUseIndex
  long servoStopTics, servoFullSpeedTics
  long negPwmResolution
  
  byte servoFlag
            
PUB StartEncoders(encoderPinPtr, encoderPtr, speedPtr, enablePinPtr, positivePinPtr_, {
  } negativePinPtr_, targetPtr_, directionFlagPtr_) | localIndex
'' This method expects encoder pins to be stored sequentially
'' in an array of bytes at encoderPinPtr.
'' The value "encoderPtr" should be the memory location
'' of four sequential longs where incremented encoder values
'' will be written.
'' The value "speedPtr" should be the memory location
'' of four sequential longs where the sum of "speedBuffer" values
'' will be written.
'' This method needs to be called to start using object.
'' There isn't a "Stop" method so multiple calls
'' will launch multiple cogs which will likely cause serious
'' problems for parent program.

  servoStopTics := (clkfreq / 1_000_000) * SERVO_STOP
  servoFullSpeedTics := (clkfreq / 1_000_000) * SERVO_FULL_SPEED
  targetPtr := targetPtr_
  negativePinPtr := negativePinPtr_
  positivePinPtr := positivePinPtr_
  directionFlagPtr0 := directionFlagPtr_
  directionFlagPtr1 := directionFlagPtr0 + 4
  directionFlagPtr2 := directionFlagPtr1 + 4
  directionFlagPtr3 := directionFlagPtr2 + 4
  
  encoderPin0 := byte[encoderPinPtr] 
  encoderPin1 := byte[encoderPinPtr][1] 
  encoderPin2 := byte[encoderPinPtr][2]
  encoderPin3 := byte[encoderPinPtr][3]
  timeAddress := @debugTime
  buffer0 := @speedBuffer       ' first location in each buffer (static)
  buffer1 := buffer0 + (SPEED_BUFFER_SIZE * 4)
  buffer2 := buffer1 + (SPEED_BUFFER_SIZE * 4)
  buffer3 := buffer2 + (SPEED_BUFFER_SIZE * 4)

  headRollover := BUFFER_SIZE_ENCODER_TIME * 4

  pulsePtr0 := @pulseBuffer
  pulsePtr1 := pulsePtr0 + (headRollover)
  pulsePtr2 := pulsePtr1 + (headRollover)
  pulsePtr3 := pulsePtr2 + (headRollover)
  
  buffer0Head := buffer0        ' current location in each buffer (dynamic)
  buffer1Head := buffer1
  buffer2Head := buffer2
  buffer3Head := buffer3
  {pulseHead0 := BUFFER_SIZE_ENCODER_TIME  * 4'pulse0
  pulseHead1 := BUFFER_SIZE_ENCODER_TIME  * 4'pulse1
  pulseHead2 := BUFFER_SIZE_ENCODER_TIME  * 4'pulse2
  pulseHead3 := BUFFER_SIZE_ENCODER_TIME  * 4'pulse3}
  pulseHead0 := 0'pulse0
  pulseHead1 := 0'pulse1
  pulseHead2 := 0'pulse2
  pulseHead3 := 0'pulse3
   
  speedBufferSumPtr0 := speedPtr     ' **** delete speedBufferSumPtrx
  speedBufferSumPtr1 := speedBufferSumPtr0 + 4
  speedBufferSumPtr2 := speedBufferSumPtr1 + 4
  speedBufferSumPtr3 := speedBufferSumPtr2 + 4
  pulseBufferSumPtr0 := speedBufferSumPtr3 + 4
  pulseBufferSumPtr1 := pulseBufferSumPtr0 + 4
  pulseBufferSumPtr2 := pulseBufferSumPtr1 + 4
  pulseBufferSumPtr3 := pulseBufferSumPtr2 + 4
  lastUpdatedPtr0 := pulseBufferSumPtr3 + 4
  lastUpdatedPtr1 := lastUpdatedPtr0 + 4
  lastUpdatedPtr2 := lastUpdatedPtr1 + 4
  lastUpdatedPtr3 := lastUpdatedPtr2 + 4

  repeat localIndex from 0 to MAX_ENCODER_INDEX 
    result := long[enablePinPtr][localIndex]
    if result == -1
      next
    largestMotorsInUseIndex := localIndex  
    enableMask0[localIndex] := 1 << result
    'dira[result] := 1
    'outa[result] := 1
    result := long[positivePinPtr][localIndex]
    if result == -1
      servoFlag := 1
      next
    dira[result] := 1
    if result <> -1
      dira[long[negativePinPtr][localIndex]] := 1
    
   { forwardMask0[localIndex] := long[positivePinPtr][localIndex]
    result := long[negativePinPtr][localIndex]
    if result == -1
      next
    reverseMask0[localIndex] := 1 << result    }
  
    
  SetFrequency(DEFAULT_PWM_FREQUENCY)
  SetResolution(DEFALUT_PWM_RESOLUTION)
  
  onTimePtr0 := @pwmOnTime
  onTimePtr1 := onTimePtr0 + 4
  onTimePtr2 := onTimePtr1 + 4
  onTimePtr3 := onTimePtr2 + 4
  pwmTimePtr := @pwmTime
  pwmTimeCog := pwmTime  
  result := cognew(@enter, encoderPtr)

PUB SetFrequency(localFrequency)

  if servoFlag
    pwmFrequency := SERVO_FREQUENCY
  else
    pwmFrequency := localFrequency
  pwmTime := clkfreq / pwmFrequency
  
PUB SetResolution(localResolution)

  pwmResolution := localResolution
  negPwmResolution := -1 * pwmResolution
  
PUB RefreshSpeed | localSpeed, localIndex

  result := @pwmOnTime

  if servoFlag
    repeat localIndex from 0 to largestMotorsInUseIndex
      localSpeed := long[targetPtr][localIndex]
      localSpeed <#= pwmResolution
      localSpeed #>= negPwmResolution
      pwmOnTime[localIndex] := servoStopTics + ((servoFullSpeedTics * localSpeed) / pwmResolution) 
  else
    repeat localIndex from 0 to largestMotorsInUseIndex
      localSpeed := long[targetPtr][localIndex]
      if localSpeed == 0
        outa[long[positivePinPtr][localIndex]] := 0
        if outa[long[negativePinPtr][localIndex]] <> -1
          outa[long[negativePinPtr][localIndex]] := 0
        pwmOnTime[localIndex] := 0
        next  
      elseif localSpeed > 0
        if outa[long[negativePinPtr][localIndex]] <> -1
          outa[long[negativePinPtr][localIndex]] := 0
        outa[long[positivePinPtr][localIndex]] := 1
      else
        -localSpeed
        outa[long[positivePinPtr][localIndex]] := 0
        if outa[long[negativePinPtr][localIndex]] <> -1
          outa[long[negativePinPtr][localIndex]] := 1  
      localSpeed <#= pwmResolution
      pwmOnTime[localIndex] := (pwmTime * localSpeed) / pwmResolution
 
PUB GetObjectName
'' This method returns the name of this object.
'' This method is not needed for proper operation of object
'' and may be deleted.

  result := @objectName

PUB GetDebugTime
'' This method returns the value of "debugTime" to aid
'' in debugging program.
'' This method is not needed for proper operation of object
'' and may be deleted.

  result := debugTime
  
PUB GetBufferLocation
'' This method returns the location of "speedBuffer" to aid
'' in debugging program.
'' This method is not needed for proper operation of object
'' and may be deleted.

  result := @speedBuffer
  
DAT                     org       0

enter                   mov     encoderAddress, par 
                        mov     encoderAddress0, encoderAddress 
                        mov     encoderAddress1, encoderAddress0
                        add     encoderAddress1, #4 
                        mov     encoderAddress2, encoderAddress1
                        add     encoderAddress2, #4 
                        mov     encoderAddress3, encoderAddress2
                        add     encoderAddress3, #4 

                        or      dira, enableMask0
                        or      dira, enableMask1
                        'or      dira, enableMask2
                       ' or      dira, enableMask3
                       { or      dira, forwardMask0
                        or      dira, forwardMask1
                        or      dira, forwardMask2
                        or      dira, forwardMask3
                        or      dira, reverseMask0
                        or      dira, reverseMask1
                        or      dira, reverseMask2
                        or      dira, reverseMask3  }
                           
                        mov     loopsTilWrite, loopsToWrite                 
                        mov     headCountDown, bufferSize
                        mov     oldscan0, ina                   ' get start-up inputs
                        mov     oldscan1, oldscan0                    
                        mov     oldscan2, oldscan0
                        mov     oldscan3, oldscan0                                            
                        shr     oldscan0, encoderPin0           ' z-align scan   
                        and     oldscan0, #%11                  ' clear other bits
                        shr     oldscan1, encoderPin1           ' z-align scan   
                        and     oldscan1, #%11                  ' clear other bits
                        shr     oldscan2, encoderPin2           ' z-align scan   
                        and     oldscan2, #%11                  ' clear other bits
                        shr     oldscan3, encoderPin3           ' z-align scan
                        and     oldscan3, #%11                  ' clear other bits
                        
                        mov     timer, cnt                      ' sync with system cnt
                        mov     pwmTimer, timer
                        'add     timer, enctiming                ' set loop timing                        
                        mov     speedTimer, timer

'-------------------------------------------------------------------------------

encloop                 mov     nowTime, cnt    ' use the same time point for all comparisons
                                         ' if source is larger, write c

                        mov     pwmTimeTemp, nowTime 
                        subs    pwmTimeTemp, pwmTimer
                        'mov     pwmTimeTemp, nowTime 
                        'subs    pwmTimeTemp, pwmTimer
                        cmps    onTimeCog0, pwmTimeTemp wc 
              if_c      andn    outa, enableMask0
                        cmps    onTimeCog1, pwmTimeTemp wc 
              if_c      andn    outa, enableMask1
             {           cmps    onTimeCog2, pwmTimeTemp wc 
              if_c      andn    outa, enableMask2
                        cmps    onTimeCog3, pwmTimeTemp wc 
              if_c      andn    outa, enableMask3       }
                        cmps    pwmTimeCog, pwmTimeTemp wc
              if_c      jmp    #checkPwm
                        
              
continueLoop          '  subs    nowTime, timer                        
                      '  cmps    enctiming, nowTime wc
           '   if_nc     jmp    #encloop 'checkEncoders
                        



                        rdlong  onTimeCog0, onTimePtr0
                        rdlong  onTimeCog1, onTimePtr1
                       ' rdlong  onTimeCog2, onTimePtr2
                       ' rdlong  onTimeCog3, onTimePtr3
                       { rdlong  directionCog0, directionPtr0
                        rdlong  directionCog1, directionPtr1
                        rdlong  directionCog2, directionPtr2
                        rdlong  directionCog3, directionPtr3}
                        rdlong  pwmTimeCog, pwmTimePtr
checkEncoders           add     timer, enctiming                ' hold for loop timing
                        wrlong  timer, timeAddress             ' This should be at least
                                                                ' as long as the longest
                                                                ' possible loop will take
                                                                ' 4 + (4 * 30) + (4 * 9) +
                                                                ' 50 = 120 instructions =
                                                                ' 480 clock cycles
scan                  '  djnz    loopsTilWrite, #continueScan
                       ' call    #refreshCount
continueScan            mov     originalScan, ina               ' get current inputs

                        mov     newscan, originalScan           
                        shr     newscan, encoderPin0
                        mov     oldScan, oldscan0               
                        mov     encoderTotal, encoderTotal0     
                        mov     encoderAddress, encoderAddress0
                                
                        mov     lastPulse, lastPulse0
                        mov     pulseBufferSumPtr, pulseBufferSumPtr0  ' no change
                        mov     pulseHead, pulseHead0   ' will change
                        mov     lastUpdatedPtr, lastUpdatedPtr0    ' no change
                        mov     pulseTotal, pulseTotal0   ' will change
                        mov     pulsePtr, pulsePtr0       ' no change
                        mov     directionFlagPtr, directionFlagPtr0

                        call    #chkEncoder                     ' run alrorithm with active variables
                        mov     oldscan0, newscan 
                        mov     encoderTotal0, encoderTotal
                        add     encoderQuanta0, encoderChange

                        '**
                        mov     pulseTotal0, pulseTotal
                        mov     pulseHead0, pulseHead
                        mov     lastPulse0, lastPulse
                                                                                                
                        mov     newscan, originalScan           
                        shr     newscan, encoderPin1
                        mov     oldScan, oldscan1               
                        mov     encoderTotal, encoderTotal1     
                        mov     encoderAddress, encoderAddress1   

                        mov     lastPulse, lastPulse1
                        mov     pulseBufferSumPtr, pulseBufferSumPtr1  ' no change
                        mov     pulseHead, pulseHead1   ' will change
                        mov     lastUpdatedPtr, lastUpdatedPtr1    ' no change
                        mov     pulseTotal, pulseTotal1   ' will change
                        mov     pulsePtr, pulsePtr1       ' no change
                        mov     directionFlagPtr, directionFlagPtr1
                        
                        call    #chkEncoder                     ' run alrorithm with active variables
                        mov     oldscan1, newscan 
                        mov     encoderTotal1, encoderTotal
                        add     encoderQuanta1, encoderChange

                        mov     pulseTotal1, pulseTotal
                        mov     pulseHead1, pulseHead
                        mov     lastPulse1, lastPulse
                        
                        mov     newscan, originalScan           
                        shr     newscan, encoderPin2
                        mov     oldScan, oldscan2               
                        mov     encoderTotal, encoderTotal2     
                        mov     encoderAddress, encoderAddress2   

                        mov     lastPulse, lastPulse2
                        mov     pulseBufferSumPtr, pulseBufferSumPtr2  ' no change
                        mov     pulseHead, pulseHead2   ' will change
                        mov     lastUpdatedPtr, lastUpdatedPtr2    ' no change
                        mov     pulseTotal, pulseTotal2   ' will change
                        mov     pulsePtr, pulsePtr2       ' no change
                        mov     directionFlagPtr, directionFlagPtr2
                        
                        call    #chkEncoder                     ' run alrorithm with active variables
                        mov     oldscan2, newscan 
                        mov     encoderTotal2, encoderTotal
                        add     encoderQuanta2, encoderChange

                        mov     pulseTotal2, pulseTotal
                        mov     pulseHead2, pulseHead
                        mov     lastPulse2, lastPulse
                        
                        mov     newscan, originalScan           
                        shr     newscan, encoderPin3
                        mov     oldScan, oldscan3               
                        mov     encoderTotal, encoderTotal3     
                        mov     encoderAddress, encoderAddress3   

                        mov     lastPulse, lastPulse3
                        mov     pulseBufferSumPtr, pulseBufferSumPtr3  ' no change
                        mov     pulseHead, pulseHead3   ' will change
                        mov     lastUpdatedPtr, lastUpdatedPtr3    ' no change
                        mov     pulseTotal, pulseTotal3   ' will change
                        mov     pulsePtr, pulsePtr3       ' no change
                        mov     directionFlagPtr, directionFlagPtr3
                        
                        call    #chkEncoder                     ' run alrorithm with active variables
                        mov     oldscan3, newscan 
                        mov     encoderTotal3, encoderTotal
                        add     encoderQuanta3, encoderChange

                        mov     pulseTotal3, pulseTotal
                        mov     pulseHead3, pulseHead
                        mov     lastPulse3, lastPulse
                        
                        jmp     #encloop

'-------------------------------------------------------------------------------
                        
chkencoder              mov     encoderChange, zero
                        and     newscan, #%11                   ' about 30 instructions
                        cmp     newscan, oldScan        wz      ' check for change
              if_z      jmp     #chkencoder_ret 'encloop 'chkencoder ' ' wait if none

                        'there is a change
                        
                        add     pulsePtr, pulseHead
                        mov     newPulse, cnt
                        rdlong  oldPulse, pulsePtr
                        sub     pulseTotal, oldPulse
                        wrlong  newPulse, lastUpdatedPtr
                        'sub     lastPulse, newPulse  ' lastPulse is differnence between loop's cnt values
                        mov     temp, newPulse ' store this loop's cnt value in lastPulse

                        sub     temp, lastPulse
                        mov     lastPulse, newPulse
                        'shr     temp, #4
                        add     pulseTotal, temp
                        'wrlong  pulseTotal, pulseBufferSumPtr   ' yes stomping
                        mov     tempTotal, pulseTotal
                        shr     tempTotal, #BITS_TO_AVERAGE_ENCODER_TIME
                        wrlong  tempTotal, pulseBufferSumPtr   ' yes stomping
                        
                                     
                        add     pulseHead, #4
                        cmpsub  pulseHead, headRollover                
                        
                        wrlong  temp, pulsePtr                  ' not stomping
                        
                        
case11                  cmp     oldscan, #%11           wz      ' oldscan == %11?         
              if_nz     jmp     #case01                         '20 if no, try next         
                        cmp     newscan, #%01           wz      ' clockwise (postive)? Z = yes 
                        jmp     #update

case01                  cmp     oldscan, #%01           wz
              if_nz     jmp     #case00
                        cmp     newscan, #%00           wz   '40
                        jmp     #update                        

case00                  cmp     oldscan, #%00           wz      
              if_nz     jmp     #case10                         
                        cmp     newscan, #%10           wz      
                        jmp     #update                      '60
                                                                          
case10                  cmp     oldscan, #%10           wz
              if_nz     jmp     #chkencoder_ret                 ' (should never happen)
                        cmp     newscan, #%11           wz

update        if_z      mov     encoderChange, one                ' increment value

decvalue      if_nz     mov     encoderChange, negOne              '80 decrement
                        wrlong  encoderChange, directionFlagPtr
                        add     encoderTotal, encoderChange
writeTotal              wrlong  encoderTotal, encoderAddress' 88 to 92update hub  ' keep this
                                               
chkencoder_ret          ret
                       
'-------------------------------------------------------------------------------
refreshCount            mov     loopsTilWrite, loopsToWrite
                        'rdlong  oldQuanta, buffer0Head
                        sub     bufferTotal0, oldQuanta
                        add     bufferTotal0, encoderQuanta0
                        'wrlong  encoderQuanta0, buffer0Head       ' not stomping
                        add     buffer0Head, #4                   ' use #1 if buffer is byte size
                        mov     encoderQuanta0, #0                ' clear the encoder total
                        'wrlong  bufferTotal0, speedBufferSumPtr0  ' not stomping

                        'rdlong  oldQuanta, buffer1Head
                        sub     bufferTotal1, oldQuanta
                        add     bufferTotal1, encoderQuanta1
                        'wrlong  encoderQuanta1, buffer1Head
                        add     buffer1Head, #4                   ' use #1 if buffer is byte size
                        mov     encoderQuanta1, #0
                        'wrlong  bufferTotal1, speedBufferSumPtr1
 
                        'rdlong  oldQuanta, buffer2Head
                        sub     bufferTotal2, oldQuanta
                        add     bufferTotal2, encoderQuanta2
                        'wrlong  encoderQuanta2, buffer2Head
                        add     buffer2Head, #4                   ' use #1 if buffer is byte size
                        mov     encoderQuanta2, #0
                        'wrlong  bufferTotal2, speedBufferSumPtr2
 
                        'rdlong  oldQuanta, buffer3Head
                        sub     bufferTotal3, oldQuanta
                        add     bufferTotal3, encoderQuanta3
                        'wrlong  encoderQuanta3, buffer3Head
                        add     buffer3Head, #4                   ' use #1 if buffer is byte size
                        mov     encoderQuanta3, #0
                        'wrlong  bufferTotal3, speedBufferSumPtr3
 
                        
                        djnz    headCountDown, #refreshCount_ret
                        call    #refreshHead
refreshCount_ret        ret
'-------------------------------------------------------------------------------
refreshHead             mov     buffer0Head, buffer0
                        mov     buffer1Head, buffer1
                        mov     buffer2Head, buffer2
                        mov     buffer3Head, buffer3   
                        mov     headCountDown, bufferSize
refreshHead_ret         ret

'-------------------------------------------------------------------------------
checkPwm                add     pwmTimer, pwmTimeCog
                        tjz     onTimeCog0, #checkPwm1                        
                        or      outa, enableMask0
checkPwm1               tjz     onTimeCog1, #checkPwm2
                        or      outa, enableMask1
checkPwm2               tjz     onTimeCog2, #checkPwm3
                        or      outa, enableMask2
checkPwm3               tjz     onTimeCog3, #continueLoop ' checkPwm_ret
                        or      outa, enableMask3    

                        jmp     #continueLoop
'checkPwm_ret           ret   
'-------------------------------------------------------------------------------

{
zeroSpeed               andn    outa, enableMask0
                        andn    outa, enableMask0
                        andn    outa, enableMask0
 }
'-------------------------------------------------------------------------------


'-------------------------------------------------------------------------------


'-------------------------------------------------------------------------------

zero                    long 0
one                     long 1
negOne                  long -1
enctiming               long CYCLES_PER_ENCODER_CHECK '800 '20000 '60_000 '30_000 '3200 '1600 '        '
bufferSize              long SPEED_BUFFER_SIZE
loopsToWrite            long ENCODER_LOOPS_PER_WRITE
'speedInterval           long _SpeedInterval
'timeQuanta              long _SpeedInterval / 512 '255
'maxSpeed                long 511 '255
'minSpeed                long -511 '255
'pGain                   long _PGain
'pGainMask               long 0
'bufferAndMask           long 0
timeAddress             long 0
encoderAddress          long 0
encoderTotal            long 0 
encoderTotal0           long 0 
encoderTotal1           long 0 
encoderTotal2           long 0 
encoderTotal3           long 0
encoderChange           long 0
encoderQuanta0          long 0
encoderQuanta1          long 0
encoderQuanta2          long 0
encoderQuanta3          long 0
previousEncoderTotal0   long 0 
previousEncoderTotal1   long 0 
previousEncoderTotal2   long 0 
previousEncoderTotal3   long 0

encoderPin0             long 0
encoderPin1             long 0
encoderPin2             long 0
encoderPin3             long 0

' first location in each buffer
buffer0                 long 0
buffer1                 long 0
buffer2                 long 0
buffer3                 long 0
pulsePtr                long 0
pulsePtr0               long 0
pulsePtr1               long 0
pulsePtr2               long 0
pulsePtr3               long 0

' current location in each buffer
buffer0Head             long 0
buffer1Head             long 0
buffer2Head             long 0
buffer3Head             long 0
pulseHead               long 0
pulseHead0              long 0
pulseHead1              long 0
pulseHead2              long 0
pulseHead3              long 0

tempHead                long 0
bufferTotal0            long 0
bufferTotal1            long 0
bufferTotal2            long 0
bufferTotal3            long 0
pulseTotal              long 0
pulseTotal0             long 0 
pulseTotal1             long 0
pulseTotal2             long 0
pulseTotal3             long 0
speedBufferSumPtr0      long 0    ' **** delete
speedBufferSumPtr1      long 0
speedBufferSumPtr2      long 0
speedBufferSumPtr3      long 0
pulseBufferSumPtr       long 0
pulseBufferSumPtr0      long 0
pulseBufferSumPtr1      long 0
pulseBufferSumPtr2      long 0
pulseBufferSumPtr3      long 0
lastUpdatedPtr          long 0
lastUpdatedPtr0         long 0
lastUpdatedPtr1         long 0
lastUpdatedPtr2         long 0
lastUpdatedPtr3         long 0
headRollover            long 0
enableMask0             long 0
enableMask1             long 0
enableMask2             long 0
enableMask3             long 0
{forwardMask0            long 0
forwardMask1            long 0
forwardMask2            long 0
forwardMask3            long 0
reverseMask0            long 0
reverseMask1            long 0
reverseMask2            long 0
reverseMask3            long 0 }
onTimePtr0              long 0
onTimePtr1              long 0
onTimePtr2              long 0
onTimePtr3              long 0
pwmTimePtr              long 0
pwmTimeCog              long 0
directionFlagPtr        long 0
directionFlagPtr0       long 0
directionFlagPtr1       long 0
directionFlagPtr2       long 0
directionFlagPtr3       long 0
  

'bufferHead              long 0
'oldSpeed                long 0
'newSpeed                long 0
'bufferBaseAddress       long 0
'debugCount              long 0
'encoderCount            long 0

'timeBufferOffset        long 0
'bufferPtr               res 1
encoderAddress0         res 1
encoderAddress1         res 1
encoderAddress2         res 1
encoderAddress3         res 1
oldScan                 res 1
oldscan0                res 1
oldscan1                res 1
oldscan2                res 1
oldscan3                res 1
timeCount               res 1
timer                   res 1
speedTimer              res 1
speedTimerTemp          res 1

originalScan            res 1
newscan                 res 1

difference              res 1
oldQuanta               res 1
absoluteTarget          res 1
timePtr                 res 1
headCountDown           res 1
loopsTilWrite           res 1
oldPulse                res 1
temp                    res 1
lastPulse               res 1  
lastPulse0              res 1
lastPulse1              res 1
lastPulse2              res 1
lastPulse3              res 1
newPulse                res 1
onTimeCog0              res 1
onTimeCog1              res 1
onTimeCog2              res 1
onTimeCog3              res 1
pwmTimer                res 1
pwmTimeTemp             res 1
nowTime                 res 1
tempTotal               res 1
              fit
              
DAT '' License
{{
*********************************************************************************************
*                             TERMS OF USE: MIT License                                     *                                                                   
*********************************************************************************************
* Permission is hereby granted, free of charge, to any person obtaining a copy of this      *
* software and associated documentation files (the "Software"), to deal in the Software     *
* without restriction, including without limitation the rights to use, copy, modify,        *
* merge, publish, distribute, sublicense, and/or sell copies of the Software, and to        *
* permit persons to whom the Software is furnished to do so, subject to the following       *
* conditions:                                                                               *
*                                                                                           *                                    
* The above copyright notice and this permission notice shall be included in all copies or  *
* substantial portions of the Software.                                                     *
*                                                                                           *                                   
* The software is provided "as is", without warranty of any kind, express or implied,       *
* including but not limited to the warranties of merchantability, fitness for a particular  *
* and noninfringment. In no event shall the authors or copyright holders be liable for any  *
* claim, damages or other liability, whether in an action of contract, tort or otherwise,   *
* arising from, of or in connection with the software of the use of other dealings in the   *
* software.                                                                                 *
*********************************************************************************************
}}        